/*
 * $Id: ll.c,v 1.1.1.1 2002/11/28 08:51:56 eli7 Exp $
 */

/*  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

static char *rcsid =
        "$Id: ll.c,v 1.1.1.1 2002/11/28 08:51:56 eli7 Exp $";

#include "err.h"
#include "ll.h"
#include <stdlib.h>



int
ll_init(ll_t *dummy)
{
	if (dummy == NULL) {
		BUG();
		return -1;
	}
	memset(dummy, 0, sizeof(dummy));
	return 0;
}

int 
ll_data(ll_t *this, void **data)
{
	if (data == NULL) {
		return -1;
	}
	if (this->next) {
		*data = this->next->data;
		return 0;
	}
	*data = NULL;
	return -1;
}

ll_t*
ll_next(ll_t *this)
{
	if (this) {
		return this->next;
	} else {
		return NULL;
	}
}

ll_t*
ll_find(ll_t *this, cmpfunc_t cmp, void *data)
{
	void *odata;
	while (ll_data(this, &odata) == 0) {
		if (cmp(odata, data) == 0) {
			return this;
		}
		this = ll_next(this);
	}
	return NULL;
}

int 
ll_insert(ll_t *this, void *data)
{
	ll_t *ll_new;	
	if ( (ll_new = malloc(sizeof(ll_new))) == NULL) {
		BUG();
		return -1;
	}
	ll_new->data = data;
	ll_new->next = this->next;
	this->next = ll_new;
	return 0;
}

int
ll_delete(ll_t *this, void **data)
{
	ll_t *old;
	if (this == NULL) {
		BUG();
		return -1;
	}
	if ( (old = this->next) == NULL) {
		BUGF("empty");
		return -1;
	}
	this->next = old->next;
	if (data != NULL) {
		*data = old->data;
	}
	free(old);
	return 0;
}

void 
ll_die(ll_t *this) 
{
	while (ll_delete(this, NULL) != -1) {
	}
}

int 
ll_iterate(ll_t *this, iteratorfunc_t iterator, void *args) 
{
	void *data;
	ll_t *next;
	int n = 0;
	
	if (this == NULL) {
		return -1;
	}
	while ( (next = ll_next(this)) != NULL) {
		n++;
		if (ll_data(this, &data) == -1) {
			data = NULL;
		}
		if (iterator(data, args) == -1) {
			break;
		}
		this = next;
	}
	return n;
}
